#!/usr/bin/env perl

# any2seqxml
# Copyright Dave Messina <dmessina@cpan.org>
# First version: 2010-01-15
#
# For questions, help, support, or suggestions:
# please email info@orthoxml.org
#
# For more information on SeqXML:
# please visit http://seqxml.org

use strict;
use warnings;
use Bio::SeqIO;
use Getopt::Long;
use File::Basename;
use File::Spec;

my ( $format, $species, $taxid, $source, $sourceVersion );
GetOptions(
    "format=s"        => \$format,
    "species=s"       => \$species,
    "taxid=s"         => \$taxid,
    "source=s"        => \$source,
    "sourceVersion=s" => \$sourceVersion,
);

my $usage = "
any2seqxml - convert (almost) any sequence file to SeqXML

Usage: any2seqxml --format <format> seqfile1 seqfile2 ... seqfileN

Required Parameters:
--format        <format>   the format of the input sequence

Options:
--source        <string>   the source for all of the records, usually
                           the name of a database, like \"Ensembl\".
            
--sourceVersion <string>   the version of the source. For example,
                           \"56\" or \"release11\".

Examples:
any2seqxml --format genbank mynewdata.gb
any2seqxml --format fasta my_new_data.fa my_old_data.fa
any2seqxml --format fasta --source \"UniProt\" --sourceVersion \"15.12\" uniprot.fa

More Information:
The output filename will be the same as the input filename
except the suffix will be '.xml'.

Supported input formats are anything BioPerl supports.
Here are a few common ones:
    fasta
    genbank
    embl
    interpro
    pir
    swiss

For the complete list of supported formats,
    see http://www.bioperl.org/wiki/HOWTO:SeqIO
For more info on SeqXML, see http://seqxml.org
";
die $usage unless ( $format and @ARGV );
if ( $species or $taxid ) {
    die( "must set both species and taxid if you set one!\n", $usage )
      unless ( $species and $taxid );
}

foreach my $infile (@ARGV) {

    # input stream
    my $in = Bio::SeqIO->new(
        '-format' => $format,
        '-file'   => $infile
    );

    # create output filename
    # will be the same as the input filename except the suffix will
    # be .xml
    my ( $base, $path, $suffix ) = fileparse( $infile, qr/\.[^.]*/ );
    my $newbase = $base . '.xml';
    my $outfile = File::Spec->catfile( $path, $newbase );

    # output stream
    my $out;
    if ( $source or $sourceVersion ) {

        # in case user sets only one of
        # source and sourceVersion,
        # set the other to empty string
        $source        = '' unless $source;
        $sourceVersion = '' unless $sourceVersion;
        $out           = Bio::SeqIO->new(
            '-format'        => 'seqxml',
            '-file'          => ">$outfile",
            '-source'        => $source,
            '-sourceVersion' => $sourceVersion,
        );
    }
    else {
        $out = Bio::SeqIO->new(
            '-format' => 'seqxml',
            '-file'   => ">$outfile",
        );
    }

    # main loop
    # each time, one seq record is read from the input
    # and written to the output
    while ( my $inseq_obj = $in->next_seq ) {

        # set species info if provided
        if ( $taxid && $species ) {
            my $species_obj = Bio::Species->new( '-ncbi_taxid' => $taxid );
            $species_obj->binomial($species);
            $inseq_obj->species($species_obj);
        }

        $out->write_seq($inseq_obj);

    }
}
